package eu.jonahbauer.tichu.common.model.combinations;

import eu.jonahbauer.tichu.common.model.Card;
import eu.jonahbauer.tichu.common.model.CombinationType;
import eu.jonahbauer.tichu.common.model.Stack;
import org.jetbrains.annotations.NotNull;

import java.util.List;

import static eu.jonahbauer.tichu.common.model.CombinationType.BOMB_SEQUENCE;

/**
 * A combination of type {@link CombinationType#BOMB_SEQUENCE}
 */
public final class BombSequence extends Sequence implements Bomb {
    public static BombSequence of(@NotNull List<@NotNull Card> cards) {
        return new BombSequence(cards);
    }

    BombSequence(@NotNull List<@NotNull Card> cards) {
        super(BOMB_SEQUENCE, cards, 1);
    }

    @Override
    public boolean isCompatibleWith(@NotNull Stack stack) {
        return Bomb.super.isCompatibleWith(stack);
    }

    @Override
    public boolean isHigherThan(@NotNull Stack stack) {
        assert isCompatibleWith(stack);
        return stack.isEmpty()
               || stack.top().type() != BOMB_SEQUENCE
               || size() > stack.top().size()
               || start() > ((BombSequence) stack.top()).start();
    }
}
