package eu.jonahbauer.tichu.common.model.combinations;

import eu.jonahbauer.tichu.common.model.Card;
import eu.jonahbauer.tichu.common.model.CombinationType;
import eu.jonahbauer.tichu.common.model.Stack;
import org.jetbrains.annotations.NotNull;

import java.util.List;
import java.util.NoSuchElementException;

public abstract sealed class Sequence extends AbstractCombination permits SingleSequence, PairSequence, BombSequence {
    private final int start;
    private final int length;

    Sequence(@NotNull CombinationType type, @NotNull List<@NotNull Card> cards, int n) {
        super(type, cards);
        this.length = cards.size() / n;
        this.start = getStartValue(cards, n);
    }

    /**
     * The {@linkplain Card#value() value} of the lowest card in this sequence.
     */
    public int start() {
        return start;
    }

    /**
     * The length of this sequence.
     */
    public int length() {
        return length;
    }

    @Override
    public boolean isHigherThan(@NotNull Stack stack) {
        assert isCompatibleWith(stack);
        return stack.isEmpty() || start() > ((Sequence) stack.top()).start();
    }

    static int getStartValue(@NotNull List<@NotNull Card> cards, int n) {
        for (int i = 0, size = cards.size(); i < size; i++) {
            Integer value = cards.get(i).value();
            if (value != null) return value - (i / n);
        }
        throw new NoSuchElementException();
    }
}
